@@ -1,9 +1,9 @@
 /******************************************************************************
- * Unit:    raster            release 0.21                                    *
+ * Unit:    raster            release 0.22                                    *
  * Prototype declaration of separate classes. Normally this is not needed     *
  * to be included.                                                            *
  * licency:     GPL or LGPL                                                   *
- * Copyright: (c) 1998-2019 Jaroslav Fojtik                                   *
+ * Copyright: (c) 1998-2022 Jaroslav Fojtik                                   *
  ******************************************************************************/
 #ifndef _RAS_PROT_H_
 #define _RAS_PROT_H_
@@ -69,6 +69,12 @@ public: virtual int GetPlanes(void) const {return(4);};
 	};
 
 
+class Raster1D_4BitIDX: public Raster1D_4Bit
+{
+public: virtual int GetPlanes(void) const {return(-4);};
+};
+
+
 /** 8 bit planes */
 class Raster1D_8Bit: virtual public Raster1DAbstract
 	{
@@ -218,12 +224,12 @@ public: double Min, Max;
 	virtual DWORD GetValue1D(unsigned x) const;
 	virtual double GetValue1Dd(unsigned x) const;
 	virtual void SetValue1D(unsigned x, DWORD NewValue);
-	void SetValue1Dd(unsigned x, double NewValue);
+	virtual void SetValue1Dd(unsigned x, double NewValue);
 #ifdef _REENTRANT
 	virtual DWORD PTR_GetValue1D(const void *RAW_Data1D, unsigned x) const;
 	virtual double PTR_GetValue1Dd(const void *RAW_Data1D, unsigned x) const;
 	virtual void PTR_SetValue1D(void *RAW_Data1D, unsigned x, DWORD NewValue) const;
-	void PTR_SetValue1Dd(void *RAW_Data1D, unsigned x, double NewValue) const;
+	virtual void PTR_SetValue1Dd(void *RAW_Data1D, unsigned x, double NewValue) const;
 #endif
 #ifdef OPTIMISE_SPEED
         virtual void Get(Raster1DAbstract &R1) const;
@@ -301,6 +307,7 @@ public:	Raster1D_8BitRGBA(int InitSize1D)  {Allocate1D(InitSize1D);};
 	virtual DWORD GetValue1D(unsigned x) const;
 	virtual void SetValue1D(unsigned x, DWORD NewValue);
 #ifdef _REENTRANT
+        virtual void PTR_SetValue1D(void *RAW_Data1D, unsigned x, DWORD NewValue) const;
 protected:
         virtual DWORD PTR_GetValue1DRAW(const void *RAW_Data1D, unsigned x) const 
             {if(x<4*Size1D) return(((BYTE *)RAW_Data1D)[x]); return(0);};
@@ -326,6 +333,7 @@ public:	Raster1D_16BitRGBA(int InitSize1D)	{Allocate1D(InitSize1D);}
 	virtual void SetValue1D(unsigned x, DWORD NewValue);	
 
 #ifdef _REENTRANT
+        virtual void PTR_SetValue1D(void *RAW_Data1D, unsigned x, DWORD NewValue) const;
 protected:
         virtual DWORD PTR_GetValue1DRAW(const void *RAW_Data1D, unsigned x) const 
             {if(x<4*Size1D) return(((WORD *)RAW_Data1D)[x]);return(0);};
