@@ -1,10 +1,10 @@
 /*************************************************************
- * Unit:    raster            release 0.20                   *
+ * Unit:    raster            release 0.22                   *
  * Purpose: General manipulation n dimensional matrices      *
  *          n = 1, 2 and 3.			             *
  * Modul:   raster.cc                                        *
  * Licency: GPL or LGPL                                      *
- * Copyright: (c) 1998-2021 Jaroslav Fojtik                  *
+ * Copyright: (c) 1998-2022 Jaroslav Fojtik                  *
  *************************************************************/
 #include<stdio.h>
 #include<string.h>
@@ -114,7 +114,14 @@ int shift;
    return;
    }
 
- shift = R1.GetPlanes() - GetPlanes();
+ i = R1.GetPlanes(); 
+ shift = GetPlanes();
+#ifdef QWORD
+ if(i>32) i=32;
+ if(shift>32) shift=32;
+#endif
+
+ shift = i - shift;
  if(shift>0)
    {
    for(i=0; i<maxi; i++)
@@ -125,7 +132,7 @@ int shift;
    shift = -shift;
    for(i=0; i<maxi; i++)
      R1.SetValue1D(i,GetValue1D(i)>>shift);
-   } 
+   }
 }
 
 
@@ -137,37 +144,43 @@ unsigned i,maxi;
 signed char shift;
 
   if(Data1D==NULL || R1.Data1D==NULL) return;
-  maxi = Size1D>R1.Size1D?Size1D:R1.Size1D;
+  maxi = Size1D>R1.Size1D ? Size1D : R1.Size1D;
 
   if(GetPlanes() == R1.GetPlanes())		// No bitplane change - copy data only
-    {
+  {
     i = Size1D;
     if(R1.Size1D<i) i=R1.Size1D;
     memcpy(Data1D, R1.Data1D, ((long)i*labs(GetPlanes())+7)/8);
     return;
-    }
+  }
 
   if(GetPlanes()<0 || R1.GetPlanes()<0)		// Floating point operation
-    {
+  {
     for(i=0;i<maxi;i++)
       SetValue1Dd(i,R1.GetValue1Dd(i));
     return;
-    }
+  }
 
   shift = GetPlanes() - R1.GetPlanes();
   if(shift>0)
+  {
+    unsigned k = 0;
+    while(shift >= 0)		// Needs to replicate lower bits.
     {
-    for(i=0;i<maxi;i++)
-      SetValue1D(i,R1.GetValue1D(i)<<shift);
-    return;
+      k |= 1 << shift;    
+      shift -= R1.GetPlanes();
     }
+    for(i=0; i<maxi; i++)
+      SetValue1D(i,R1.GetValue1D(i)*k);
+    return;
+  }
   else
-    {
+  {
     shift = -shift;
-    for(i=0;i<maxi;i++)
+    for(i=0; i<maxi; i++)
       SetValue1D(i,R1.GetValue1D(i)>>shift);
     return;
-    }
+  }
 }
 
 
@@ -187,7 +200,7 @@ int shift;
    return;
    }
 
- if(R1.GetPlanes()<0 || GetPlanes()<0)	// Floating point conversion 
+ if(R1.GetPlanes()<0 || GetPlanes()<0)	// Floating point conversion
    {
    if(GetPlanes()<0)	// Both source and target are FP.
      {
@@ -454,6 +467,8 @@ BYTE Mask;
     {
     case 1: Raster1DAbstract::Get(R1);
 	    return;
+    case 4: Conv1_4((BYTE *)R1.Data1D,(const BYTE *)Data1D,Size1D);
+	    return;
     case 8: Conv1_8((BYTE *)R1.Data1D,(const BYTE *)Data1D,Size1D);
 	    return;
     case 16:Conv1_16((WORD *)R1.Data1D,(const BYTE *)Data1D,Size1D);
@@ -619,8 +634,8 @@ void Raster1D_4Bit::Get(Raster1DAbstract &R1) const
     {
     case 1: Conv4_1((BYTE *)R1.Data1D,(const BYTE *)Data1D,Size1D);
 	    return;
-    case 4: Raster1DAbstract::Get(R1);
-	    return;
+//  case 4: Raster1DAbstract::Get(R1);
+//	    return;
     case 8: Conv4_8((BYTE *)R1.Data1D,(const BYTE *)Data1D,Size1D);
 	    return;
     case 16:Conv4_16((WORD *)R1.Data1D,(const BYTE *)Data1D,Size1D);
@@ -637,8 +652,10 @@ void Raster1D_4Bit::Set(const Raster1DAbstract &R1)
 {
   switch(R1.GetPlanes())
     {
-    case 4: Raster1DAbstract::Set(R1);
+    case 1: Conv1_4((BYTE *)Data1D,(const BYTE *)R1.Data1D,Size1D);
 	    return;
+//  case 4: Raster1DAbstract::Set(R1);
+//	    return;
     case 8: Conv8_4((BYTE *)Data1D,(const BYTE *)R1.Data1D,Size1D);
 	    return;
     case 16:Conv16_4((BYTE *)Data1D,(const WORD *)R1.Data1D,Size1D);
@@ -842,6 +859,8 @@ signed char shift;
 	    return;
     case 16:Raster1DAbstract::Get(R1);
 	    return;
+    case 24:Conv16_24((BYTE *)R1.Data1D,(const WORD *)Data1D,Size1D);
+	    return;
     case 32:Conv16_32((DWORD *)R1.Data1D,(const WORD *)Data1D,Size1D);
 	    return;
     }
@@ -878,7 +897,9 @@ signed char shift;
     case 8: Conv8_16((WORD *)Data1D,(const BYTE *)R1.Data1D,Size1D);
 	    return;
     case 16:Raster1DAbstract::Set(R1);
-	    return;    
+	    return;
+    case 24:Conv24_16((WORD *)Data1D,(const BYTE *)R1.Data1D,Size1D);
+            return;    
     case 32:Conv32_16((WORD *)Data1D,(const DWORD *)R1.Data1D,Size1D);
 	    return;
     }
@@ -961,7 +982,15 @@ void Raster1D_24Bit::Get(Raster1DAbstract &R1) const
   switch(R1.GetPlanes())
     {
     case 8: Conv24_8((BYTE *)R1.Data1D,(const BYTE *)Data1D,Size1D);
-	    return;  
+	    return;
+    case 16:Conv24_16((WORD *)R1.Data1D,(const BYTE *)Data1D,Size1D);
+	    return;
+    case 32:Conv24_32((DWORD *)R1.Data1D,(const BYTE *)Data1D,Size1D);
+	    return;
+#ifdef QWORD
+    case 64:Conv24_64((QWORD *)R1.Data1D,(const BYTE *)Data1D,Size1D);
+	    return;
+#endif
     }
   Raster1DAbstract::Get(R1);
 }
@@ -975,6 +1004,10 @@ void Raster1D_24Bit::Set(const Raster1DAbstract &R1)
 	    return;
     case 8: Conv8_24((BYTE *)Data1D,(const BYTE *)R1.Data1D,Size1D);
 	    return;
+    case 16:Conv16_24((BYTE *)Data1D,(const WORD *)R1.Data1D,Size1D);
+	    return;
+    case 32:Conv32_24((BYTE *)Data1D,(const DWORD *)R1.Data1D,Size1D);
+	    return;
     }
   Raster1DAbstract::Set(R1);
 }
@@ -1097,8 +1130,15 @@ signed char shift;
 	    return;
     case 16:Conv32_16((WORD *)R1.Data1D,(const DWORD *)Data1D,Size1D);
 	    return;
+    case 24:Conv32_24((BYTE *)R1.Data1D,(const DWORD *)Data1D,Size1D);
+	    return;
     case 32:Raster1DAbstract::Get(R1);
 	    return;
+#ifdef QWORD
+    case 64:Conv32_64((QWORD *)R1.Data1D,(const DWORD*)Data1D,Size1D);
+	    return;
+#endif
+
     }
 
   const DWORD *N_Buffer32Bit = (const DWORD *)Data1D;
@@ -1136,6 +1176,8 @@ signed char shift;
 	    return;
     case 16:Conv16_32((DWORD *)Data1D,(const WORD *)R1.Data1D,Size1D);
 	    return;
+    case 24:Conv24_32((DWORD *)Data1D,(const BYTE *)R1.Data1D,Size1D);
+	    return;
     case 32:Raster1DAbstract::Set(R1);
 	    return;
     }
@@ -1166,6 +1208,26 @@ signed char shift;
 /* ------------ 64 bit planes -------------- */
 #ifdef QWORD
 
+DWORD Raster1D_64Bit::GetValue1D(unsigned x) const 
+{
+  if(x>=Size1D) return 0;
+  const QWORD q = ((QWORD *)Data1D)[x];
+  return((DWORD)(q>>32));
+};
+
+
+void Raster1D_64Bit::SetValue1D(unsigned x, DWORD NewValue)
+{
+  if(x<Size1D) 
+    ((QWORD *)Data1D)[x] = NewValue *
+#ifdef __BORLANDC__
+			(QWORD)0x0000000100000001ui64;
+#else
+			(QWORD)0x0000000100000001ll;
+#endif                       
+  return;
+};
+
 
 void Raster1D_64Bit::SetValue1Dd(unsigned x, double NewValue)
 {
@@ -1206,6 +1268,45 @@ void Raster1D_64Bit::PTR_SetValue1Dd(void *RAW_Data1D, unsigned x, double NewVal
 #endif
 
 
+#ifdef OPTIMISE_SPEED
+
+/*
+void Raster1D_64Bit::Get(Raster1DAbstract &R1) const
+{
+  switch(R1.GetPlanes())
+    {
+    case 1: Conv64_1((BYTE *)R1.Data1D,(const BYTE *)Data1D,Size1D);
+	    return;
+    case 8: Conv64_8((BYTE *)R1.Data1D,(const BYTE *)Data1D,Size1D);
+	    return;
+    case 16:Conv64_16((WORD *)R1.Data1D,(const BYTE *)Data1D,Size1D);
+	    return;
+    case 32:Conv64_32((DWORD *)R1.Data1D,(const BYTE *)Data1D,Size1D);
+	    return;
+    }
+  Raster1DAbstract::Get(R1);
+}
+*/
+
+
+void Raster1D_64Bit::Set(const Raster1DAbstract &R1)
+{
+  switch(R1.GetPlanes())
+    {
+    case 1: Conv1_64((QWORD *)Data1D,(const BYTE *)R1.Data1D,Size1D);
+	    return;
+    case 8: Conv8_64((QWORD *)Data1D,(const BYTE *)R1.Data1D,Size1D);
+	    return;
+    case 24:Conv24_64((QWORD *)Data1D,(const BYTE *)R1.Data1D,Size1D);
+	    return;
+    case 32:Conv32_64((QWORD *)Data1D,(const DWORD *)R1.Data1D,Size1D);
+	    return;
+    }
+  Raster1DAbstract::Set(R1);
+}
+
+#endif
+
 #endif
 
 
@@ -1736,6 +1837,15 @@ void Raster1D_8BitRGBA::SetValue1D(unsigned x, DWORD NewValue)
 }
 
 
+#ifdef _REENTRANT
+void Raster1D_8BitRGBA::PTR_SetValue1D(void *RAW_Data1D, unsigned x, DWORD NewValue) const
+{
+  if(Size1D<=x) return;
+  ((DWORD *)RAW_Data1D)[x] = NewValue;
+}
+#endif
+
+
 /** 16 bit planes for 4 channels R,G,B, A */
 
 
@@ -1764,6 +1874,23 @@ void Raster1D_16BitRGBA::SetValue1D(unsigned x, DWORD NewValue)
 }
 
 
+#ifdef _REENTRANT
+void Raster1D_16BitRGBA::PTR_SetValue1D(void *RAW_Data1D, unsigned x, DWORD NewValue) const
+{
+  if(Size1D<=x) return;
+  x *= 4;
+  ((WORD *)RAW_Data1D)[x] = (NewValue & 0xFF) * 0x101;
+  NewValue >>= 8;
+  ((WORD *)RAW_Data1D)[x+1] = (NewValue & 0xFF) * 0x101;
+  NewValue >>= 8;
+  ((WORD *)RAW_Data1D)[x+2] = (NewValue & 0xFF) * 0x101;
+  NewValue >>= 8;
+  ((WORD *)RAW_Data1D)[x+3] = (NewValue & 0xFF) * 0x101;
+}
+#endif
+
+
+
 /*
 void Raster1D_24Bit::Get(Raster1DAbstract &R1) const
 {
