@@ -1,9 +1,9 @@
 /***************************************************************************
- * Unit:    raster            release 0.21                                 *
+ * Unit:    raster            release 0.22                                 *
  * Purpose: Prototypes for general manipulation n dimensional matrices     *
  * Modul:   raster.cc                                                      *
  * Licency: GPL or LGPL                                                    *
- * Copyright: (c) 1998-2021 Jaroslav Fojtik                                *
+ * Copyright: (c) 1998-2022 Jaroslav Fojtik                                *
  ***************************************************************************/
 #ifndef __Rasters_h
 #define __Rasters_h
@@ -106,6 +106,22 @@ void Flip64(QWORD *d, unsigned len);
 void AbstractError(const char *Name=NULL);
 int NearAvailPlanes(int n);
 
+#ifndef ABSTRACT_ERROR
+ #ifdef CORRUPTED_ABSTRACT
+  #define ABSTRACT_ERROR(abs_name) {AbstractError(abs_name);}
+ #else
+  #define ABSTRACT_ERROR(abs_name) =0
+ #endif
+#endif
+#ifndef ABSTRACT_ERRORi
+ #ifdef CORRUPTED_ABSTRACT
+  #define ABSTRACT_ERRORi(abs_name) {AbstractError(abs_name);return(0);}
+ #else
+  #define ABSTRACT_ERRORi(abs_name) =0
+ #endif
+#endif
+
+
 /** Base class for 1D data. */
 class Raster1DAbstract
 {
@@ -125,7 +141,7 @@ public: mutable short UsageCount;
 
 	virtual DWORD GetValue1D(unsigned x) const {return(0);};
 	virtual double GetValue1Dd(unsigned x) const {return(GetValue1D(x));};
-	virtual void SetValue1D(unsigned x, DWORD NewValue) {AbstractError("SetValue1D");};
+	virtual void SetValue1D(unsigned x, DWORD NewValue) ABSTRACT_ERROR("SetValue1D");
 	virtual void SetValue1Dd(unsigned x, double NewValue) {SetValue1D(x,(DWORD)NewValue);};
 
 	virtual int GetPlanes(void) const {return(0);};
@@ -148,15 +164,15 @@ public: mutable short UsageCount;
 	virtual void Join8Bit(BYTE *Buffer8Bit, char plane8);
 
 	virtual Raster1DAbstract *GetPalette(void) {return(NULL);}
-	virtual void SetPalette(Raster1DAbstract *NewPalette) {AbstractError("SetPalette");}
+	virtual void SetPalette(Raster1DAbstract *NewPalette) {AbstractError("Raster1DAbstract::SetPalette");}
 	friend Raster1DAbstract *CreateRaster1D(unsigned SizeX, int Planes);	
 
 protected:
 #ifdef _REENTRANT
         virtual DWORD PTR_GetValue1D(const void *RAW_Data1D, unsigned x) const {return(0);};
 	virtual double PTR_GetValue1Dd(const void *RAW_Data1D, unsigned x) const {return(PTR_GetValue1D(RAW_Data1D,x));};
-	virtual void PTR_SetValue1D(void *RAW_Data1D, unsigned x, DWORD NewValue) const {AbstractError("RAW_SetValue1D");};
-	virtual void PTR_SetValue1Dd(void *RAW_Data1D, unsigned x, double NewValue) const {AbstractError("RAW_SetValue1Dd");};
+	virtual void PTR_SetValue1D(void *RAW_Data1D, unsigned x, DWORD NewValue) const = 0; // {AbstractError("RAW_SetValue1D");};
+	virtual void PTR_SetValue1Dd(void *RAW_Data1D, unsigned x, double NewValue) {PTR_SetValue1D(RAW_Data1D,x,NewValue);};
         virtual void PTR_Get(const void *RAW_Data1D, Raster1DAbstract &R1) const;
         virtual void PTR_Set(void *RAW_Data1D, const Raster1DAbstract &R1);
 #endif
@@ -167,8 +183,8 @@ Raster1DAbstract *CreateRaster1D(unsigned Size1D, int Planes);
 class Raster1DAbstractRGB: virtual public Raster1DAbstract
 {
 public:
-  virtual DWORD GetValue1DRAW(unsigned x) const {AbstractError("GetValue1DRAW");return 0;};
-  virtual void SetValue1DRAW(unsigned x, DWORD NewValue) {AbstractError("SetValue1DRAW");};
+  virtual DWORD GetValue1DRAW(unsigned x) const ABSTRACT_ERRORi("Raster1DAbstractRGB::GetValue1DRAW");
+  virtual void SetValue1DRAW(unsigned x, DWORD NewValue) ABSTRACT_ERROR("Raster1DAbstractRGB::SetValue1DRAW");
 
   virtual unsigned GetSize1D(void) const {return(Size1D);};
   virtual char Channels(void) const {return 3;};
@@ -187,8 +203,8 @@ public:
 
 #ifdef _REENTRANT
 protected:
-  virtual DWORD PTR_GetValue1DRAW(const void *RAW_Data1D, unsigned x) const {AbstractError("PTR_GetValue1DRAW");return 0;};
-  virtual void PTR_SetValue1DRAW(void *RAW_Data1D, unsigned x, DWORD NewValue) {AbstractError("PTR_SetValue1DRAW");};
+  virtual DWORD PTR_GetValue1DRAW(const void *RAW_Data1D, unsigned x) const = 0; // {AbstractError("PTR_GetValue1DRAW");return 0;};
+  virtual void PTR_SetValue1DRAW(void *RAW_Data1D, unsigned x, DWORD NewValue) = 0; // {AbstractError("PTR_SetValue1DRAW");};
 #endif
 };
 Raster1DAbstractRGB *CreateRaster1DRGB(unsigned Size1D, int Planes);
@@ -197,8 +213,8 @@ Raster1DAbstractRGB *CreateRaster1DRGB(unsigned Size1D, int Planes);
 class Raster1DAbstractRGBA: virtual public Raster1DAbstract
 {
 public:
-  virtual DWORD GetValue1DRAW(unsigned x) const {AbstractError("GetValue1DRAW");return 0;};
-  virtual void SetValue1DRAW(unsigned x, DWORD NewValue) {AbstractError("SetValue1DRAW");};
+  virtual DWORD GetValue1DRAW(unsigned x) const ABSTRACT_ERRORi("Raster1DAbstractRGBA::GetValue1DRAW");
+  virtual void SetValue1DRAW(unsigned x, DWORD NewValue) ABSTRACT_ERROR("Raster1DAbstractRGBA::SetValue1DRAW");
 
   virtual unsigned GetSize1D(void) const {return(Size1D);};
   virtual char Channels(void) const {return 4;};
@@ -219,8 +235,8 @@ public:
 
 #ifdef _REENTRANT
 protected:
-  virtual DWORD PTR_GetValue1DRAW(const void *RAW_Data1D, unsigned x) const {AbstractError("PTR_GetValue1DRAW");return 0;};
-  virtual void PTR_SetValue1DRAW(void *RAW_Data1D, unsigned x, DWORD NewValue) {AbstractError("PTR_SetValue1DRAW");};
+  virtual DWORD PTR_GetValue1DRAW(const void *RAW_Data1D, unsigned x) const = 0; // {AbstractError("PTR_GetValue1DRAW");return 0;};
+  virtual void PTR_SetValue1DRAW(void *RAW_Data1D, unsigned x, DWORD NewValue) = 0; // {AbstractError("PTR_SetValue1DRAW");};
 #endif
 };
 
@@ -405,6 +421,8 @@ class APalette: virtual public Raster1DAbstractRGB
 public: virtual void Get(unsigned index, RGBQuad *RGB) const;
         virtual void Set(unsigned index, const RGBQuad *RGB);
 	friend APalette *BuildPalette(unsigned Indices, char type);
+
+        virtual void SetPalette(Raster1DAbstract *NewPalette) {AbstractError("APalette::SetPalette");}
 	};
 APalette *BuildPalette(unsigned Indices, char type=0);
 
