@@ -3,7 +3,7 @@
  * function:    Miscellaneous utilities for manipulation with images.         *
  * modul:       rasterut.cc                                                   *
  * licency:     GPL or LGPL                                                   *
- * Copyright: (c) 1998-2021 Jaroslav Fojtik                                   *
+ * Copyright: (c) 1998-2022 Jaroslav Fojtik                                   *
  ******************************************************************************/
 #include<stdio.h>
 #include<string.h>
@@ -347,18 +347,22 @@ void Flip2D(Raster2DAbstract *r2D)
  if(r2D->Data2D==NULL) return;
 
 #if defined(POINTER_SIZE) && POINTER_SIZE==32
- Flip32((DWORD*)r2D->Data2D, r2D->GetSize2D());
+  Flip32((DWORD*)r2D->Data2D, r2D->GetSize2D());
 #else
- void *ptr, **P1, **P2;
- P1 = r2D->Data2D;
- P2 = P1 + (r2D->Size2D-1); 
- while(P2>P1)
-   {
-   ptr = *P1;
-   *P1 = *P2;
-   *P2 = ptr;
-   P1++; P2--;
-   }
+ #if defined(POINTER_SIZE) && POINTER_SIZE==64 && defined(QWORD)
+  Flip64((QWORD*)r2D->Data2D, r2D->GetSize2D());
+ #else
+  void *ptr, **P1, **P2;
+  P1 = r2D->Data2D;
+  P2 = P1 + (r2D->Size2D-1); 
+  while(P2>P1)
+    {
+    ptr = *P1;
+    *P1 = *P2;
+    *P2 = ptr;
+    P1++; P2--;
+    }
+ #endif
 #endif
 }
 
@@ -521,6 +525,9 @@ void Flip2D(Raster3DAbstract *r3D)
 #if defined(POINTER_SIZE) && POINTER_SIZE==32
    Flip32((DWORD*)r3D->Data3D[z], r3D->GetSize2D());
 #else
+ #if defined(POINTER_SIZE) && POINTER_SIZE==64 && defined(QWORD)
+   Flip64((QWORD*)r3D->Data3D[z], r3D->GetSize2D());
+ #else
    void *ptr, **P1, **P2;
    P1 = r3D->Data3D[z];
    P2 = r3D->Data3D[z] + (r3D->Size2D-1);
@@ -531,6 +538,7 @@ void Flip2D(Raster3DAbstract *r3D)
      *P2 = ptr;
      P1++; P2--;
      }
+ #endif
 #endif
  }
 }
@@ -539,22 +547,26 @@ void Flip2D(Raster3DAbstract *r3D)
 /** This procedure provides flipping along 3rd dimension of the raster. */
 void Flip3D(Raster3DAbstract *r3D)
 {
- if(r3D==NULL) return;
- if(r3D->Size3D==0 || r3D->Data3D==NULL) return;
+  if(r3D==NULL) return;
+  if(r3D->Size3D==0 || r3D->Data3D==NULL) return;
 
 #if defined(POINTER_SIZE) && POINTER_SIZE==32
- Flip32((DWORD*)r3D->Data3D, r3D->GetSize3D());
+  Flip32((DWORD*)r3D->Data3D, r3D->GetSize3D());
 #else
- void **ptr, ***P1, ***P2;
- P1 = r3D->Data3D;
- P2 = r3D->Data3D + (r3D->Size3D-1);
- while(P2>P1)
-   {
-   ptr = *P1;
-   *P1 = *P2;
-   *P2 = ptr;
-   P1++; P2--;
-   }
+ #if defined(POINTER_SIZE) && POINTER_SIZE==64 && defined(QWORD)
+  Flip64((QWORD*)r3D->Data3D, r3D->GetSize3D());
+ #else
+  void **ptr, ***P1, ***P2;
+  P1 = r3D->Data3D;
+  P2 = r3D->Data3D + (r3D->Size3D-1);
+  while(P2>P1)
+    {
+    ptr = *P1;
+    *P1 = *P2;
+    *P2 = ptr;
+    P1++; P2--;
+    }
+ #endif
 #endif
 }
 
@@ -648,7 +660,7 @@ return(pal);
 /** Is this Gray scale palette? */
 int GrayPalette(Raster1DAbstractRGB *Palette, int Planes)
 {
-unsigned i,j;
+unsigned i;
 unsigned PalItems;
 
  if(Palette==NULL) return(1);
@@ -665,10 +677,10 @@ unsigned PalItems;
 
  for(i=0;i<PalItems;i++)
 	{
-	j=i*255 / (PalItems-1);
-	if( (Palette->R(i)!=j) ||
-	    (Palette->G(i)!=j) ||
-	    (Palette->B(i)!=j) ) return(0);
+	const unsigned color = i*255 / (PalItems-1);
+	if( (Palette->R(i)!=color) ||
+	    (Palette->G(i)!=color) ||
+	    (Palette->B(i)!=color) ) return(0);
 	}
  return(1);
 }
@@ -677,19 +689,20 @@ unsigned PalItems;
 /** Make this palette gray. */
 int FillGray(Raster1DAbstractRGB *Palette, int Planes)
 {
-unsigned i,j;
+unsigned i;
 
  if(Palette==NULL) return(1);
- if(Planes!=0)
-   if(3*(1<<Planes) != Palette->Size1D) return(0);
-
  if(Palette->Data1D==NULL) return(0);
- for(i=0;i<Palette->Size1D/3-1;i++)
+
+ if(Planes!=0)
+   if(3*Planes*8 != Palette->GetPlanes()) return(0);
+
+ for(i=0; i<Palette->Size1D; i++)
    {
-   j=i*255 / (Palette->Size1D/3-1);
-   Palette->R(i,j);
-   Palette->G(i,j);
-   Palette->B(i,j);
+   const unsigned color = i*255 / (Palette->Size1D-1);
+   Palette->R(i,color);
+   Palette->G(i,color);
+   Palette->B(i,color);
    }
  return(1);
 }
