@@ -75,6 +75,7 @@ PS_State::PS_State(void)
   memset(&PaperBackground, 0, sizeof(PaperBackground));
   memset(&FillBackground, 0xFF, sizeof(FillBackground)); //Use white paper.
   pPalette = NULL;
+  pRaster = NULL;
 }
 
 
@@ -85,6 +86,11 @@ PS_State::~PS_State()
     if(pPalette->UsageCount--<=1) delete pPalette;
     pPalette = NULL;
   }
+  if(pRaster != NULL)
+  {
+    if(pRaster->UsageCount--<=1) delete pRaster;
+    pRaster = NULL;
+  }
 }
 
 
@@ -94,6 +100,7 @@ PS_State & PS_State::operator=(const PS_State & OrigPSS)
   PaperBackground = OrigPSS.PaperBackground;
   FontName = OrigPSS.FontName;
   AttachPalette(OrigPSS.pPalette);
+  AttachRaster(OrigPSS.pRaster);
 return *this;
 }
 
@@ -112,6 +119,20 @@ void PS_State::AttachPalette(APalette *NewPalette)
 }
 
 
+void PS_State::AttachRaster(Raster2DAbstract *NewRaster)
+{
+  if(pRaster!=NULL)			// Detach previously attached raster
+  {
+    if(pRaster->UsageCount--<=1) delete pRaster;
+    pRaster = NULL;
+  }
+  if(NewRaster!=NULL)			// Attach raster now
+  {
+    pRaster=NewRaster; pRaster->UsageCount++;
+  }
+}
+
+
 
 ////////////////////////////////////
 
@@ -291,6 +312,40 @@ void attrPalette::prepExport(PS_State *PSS) const
 }
 
 
+////////////////////////////////////
+
+
+attrRaster::attrRaster(Raster2DAbstract *iniRaster, APalette *iniPalette)
+{
+  pRaster = iniRaster;
+  if(pRaster!=NULL) pRaster->UsageCount++;
+  pPalette = iniPalette;
+  if(pPalette!=NULL) pPalette->UsageCount++;
+}
+
+
+attrRaster::~attrRaster()
+{
+  if(pRaster != NULL)
+  {
+    if(pRaster->UsageCount--<=1) delete pRaster;
+    pRaster = NULL;
+  }
+  if(pPalette != NULL)
+  {
+    if(pPalette->UsageCount--<=1) delete pPalette;
+    pPalette = NULL;
+  }
+}
+
+
+void attrRaster::prepExport(PS_State *PSS) const
+{
+  if(PSS==NULL) return;
+  PSS->AttachRaster(pRaster);
+}
+
+
 ////////////////////////////////////
 
 vecTransform::vecTransform(void)
