@@ -122,10 +122,12 @@ public:
 	~PS_State(void);
 	PS_State &operator=(const PS_State & OrigPSS);
 	void AttachPalette(APalette *NewPalette);
+        void AttachRaster(Raster2DAbstract *NewRaster);
 
 	RGB_Record PaperBackground;
 	string FontName;
 	APalette *pPalette;
+	Raster2DAbstract *pRaster;
 };
 
 
@@ -161,12 +163,21 @@ typedef enum
   ATTR_PEN,
   ATTR_BRUSH,
   ATTR_PALETTE,
+  ATTR_RASTER,
 } ATTRIB_TYPES;
 
 
 class VectorAttribute
 {
-public:
+protected:
+  mutable int RefCounter;
+
+public:  
+  void AddRef(void) {RefCounter++;}
+  void Release(void) {if(--RefCounter<=0) delete this;}
+  int getRefCounter(void) const {return RefCounter;}
+
+  VectorAttribute() {RefCounter=1;}
   virtual ~VectorAttribute() {}
   virtual void prepExport(PS_State *PSS) const = 0;
   virtual int getAttribType(void) const {return ATTR_NONE;};
@@ -242,6 +253,19 @@ public:
 };
 
 
+class attrRaster: public VectorAttribute
+{
+public:
+  APalette *pPalette;
+  Raster2DAbstract *pRaster;
+
+  attrRaster(Raster2DAbstract *iniRaster, APalette *iniPalette=NULL);
+  virtual ~attrRaster();
+  virtual void prepExport(PS_State *PSS) const;
+  virtual int getAttribType(void) const {return ATTR_PALETTE;};
+};
+
+
 class vecTransform
 {
 public:
