@@ -252,8 +252,8 @@ BYTE char_array_4[4], char_array_3[3];
 }
 
 
-#define PAPER_SIZE	210.0
-#define PagePixels 800.0
+#define PAPER_SIZE	210.0f
+#define PagePixels 800.0f
 void TconvertedPass1_HTML::ImageHTML(void)
 {
 #ifdef DEBUG
