@@ -1235,7 +1235,7 @@ int CheckFileFormat(FILE *wpd, FFormatDetect & FilForD)
 {
 int i;
 FFormatDetect iFFD;
-float Probability=-1;
+float Probability;
 
   FilForD.DocumentType = "Unknown type";
   FilForD.Converter = "";
